/**********************************************************
 * This software is provided subject to the following terms
 * and conditions, which you should read carefully before
 * using the software. Using this software indicates your
 * acceptance of these terms and conditions.  If you do not
 * agree with these terms and conditions, do not use the
 * software. 
 * 
 * Copyright (c) 2002, Agere Systems
 * All rights reserved. 
 * 
 * Redistribution and use in source or binary forms, with or
 * without modifications, are permitted provided that the
 * following conditions are met: 
 * 
 * o Redistributions of source code must retain the above
 *   copyright notice, this list of conditions and the
 *   following Disclaimer in comments in the code as well as
 *   in the documentation and/or other materials provided with
 *   the distribution.
 * o Redistributions in binary form must reproduce the above
 *   copyright notice, this list of conditions and the following
 *   Disclaimer in the documentation and/or other materials
 *   provided with the distribution.
 * o Neither the name of Agere Systems nor the names of the
 *   contributors may be used to endorse or promote products
 *   derived from this software without specific prior written
 *   permission. 
 * 
 * Disclaimer 
 * 
 * THIS SOFTWARE IS PROVIDED "AS IS" AND ANY EXPRESS OR IMPLIED
 * WARRANTIES, INCLUDING, BUT NOT LIMITED TO, INFRINGEMENT AND THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR
 * PURPOSE ARE DISCLAIMED. ANY USE, MODIFICATION OR DISTRIBUTION OF
 * THIS SOFTWARE IS SOLELY AT THE USER'S OWN RISK. IN NO EVENT SHALL
 * AGERE SYSTEMS OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT,
 * INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING,
 * BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
 * LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER
 * CAUSED AND ON ANY THEORY OF LIABILITY, INCLUDING, BUT NOT LIMITED
 * TO, CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR
 * OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE,
 * EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE. 
 * 
 **********************************************************/

#include <stdio.h>
#include <stdlib.h>
#include "fatal.h"
#include "list.h"

#define LISTSEG 40000                   /* nodes per cluster */

struct list *ListNextCons = 0;          /* next free node */
struct list *ListTmpCons = 0;           /* temporary variable for macros */

static int consSpaceSize = 0;           /* space allocated */
static int listWarningSize = 10000000;  /* size limit for warning */

struct list *refreshCons(void) {
  struct list *new;
  int i;
  
  if(sizeof(void *) < sizeof(int)) fatal("int bigger than void*");

  /* allocate space */
  new = (struct list *)MALLOC(LISTSEG * sizeof(struct list));

  /* warning ? */
  consSpaceSize += (LISTSEG * sizeof(struct list));
  if(consSpaceSize > listWarningSize) {
    fprintf(stderr, "Warning: cons space has grown to %d\n", consSpaceSize);
    listWarningSize *= 2;
  }

  /* link new nodes */
  for(i=0; i<LISTSEG; i++) {
    new[i].next = ListNextCons;
    ListNextCons = new + i;
  }

  /* return new node */
  return newCons();
}

struct list *cons(void *e, struct list *l) {
  struct list *new = newCons();
  new->elem = e;
  new->next = l;
  return new;
}

int consSpaceUsage(void) {
  return(consSpaceSize - (listSize(ListNextCons) * sizeof(struct list)));
}

int listSize(struct list *l) {
  int i = 0;

  for(; l; l=l->next) i++;
  return i;
}
