/**********************************************************
 * This software is provided subject to the following terms
 * and conditions, which you should read carefully before
 * using the software. Using this software indicates your
 * acceptance of these terms and conditions.  If you do not
 * agree with these terms and conditions, do not use the
 * software. 
 * 
 * Copyright (c) 2002, Agere Systems
 * All rights reserved. 
 * 
 * Redistribution and use in source or binary forms, with or
 * without modifications, are permitted provided that the
 * following conditions are met: 
 * 
 * o Redistributions of source code must retain the above
 *   copyright notice, this list of conditions and the
 *   following Disclaimer in comments in the code as well as
 *   in the documentation and/or other materials provided with
 *   the distribution.
 * o Redistributions in binary form must reproduce the above
 *   copyright notice, this list of conditions and the following
 *   Disclaimer in the documentation and/or other materials
 *   provided with the distribution.
 * o Neither the name of Agere Systems nor the names of the
 *   contributors may be used to endorse or promote products
 *   derived from this software without specific prior written
 *   permission. 
 * 
 * Disclaimer 
 * 
 * THIS SOFTWARE IS PROVIDED "AS IS" AND ANY EXPRESS OR IMPLIED
 * WARRANTIES, INCLUDING, BUT NOT LIMITED TO, INFRINGEMENT AND THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR
 * PURPOSE ARE DISCLAIMED. ANY USE, MODIFICATION OR DISTRIBUTION OF
 * THIS SOFTWARE IS SOLELY AT THE USER'S OWN RISK. IN NO EVENT SHALL
 * AGERE SYSTEMS OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT,
 * INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING,
 * BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
 * LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER
 * CAUSED AND ON ANY THEORY OF LIABILITY, INCLUDING, BUT NOT LIMITED
 * TO, CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR
 * OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE,
 * EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE. 
 * 
 **********************************************************/

#include <stdio.h>
#include <stdlib.h>
#include <assert.h>

#include "fatal.h"
#include "file.h"

#define BUFSIZE (1 << 20) /* 1 MB */

static char buf[BUFSIZE];
static int pos = 0;
static FILE *fd;

#define flush() { if(fwrite(buf, 1, pos, fd) != pos) fatal("fwrite failed"); pos = 0; }

void openFile(char *name) {
  fd = fopen(name, "w");
  if (!fd) fatal("fopen failed");
}

void closeFile(void) {
  assert(fd);
  flush();
  fclose(fd);
}

void writeChar(unsigned char c) {
  assert(fd);
  if(pos == BUFSIZE) flush();
  buf[pos++] = c;
}

void writeInt(int i) {
  assert(fd);
  if(pos >= BUFSIZE - 3) flush();
  buf[pos++] = i & 0xff;
  buf[pos++] = (i & 0xff00) >> 8;
  buf[pos++] = (i & 0xff0000) >> 16;
  buf[pos++] = i >> 24;
}

void writeString(char *string) {
  assert(fd);
  while(*string) {
    if(pos == BUFSIZE) flush();
    buf[pos++] = *(string++);
  }
  writeChar(0);
}
