/**********************************************************
 * This software is provided subject to the following terms
 * and conditions, which you should read carefully before
 * using the software. Using this software indicates your
 * acceptance of these terms and conditions.  If you do not
 * agree with these terms and conditions, do not use the
 * software. 
 * 
 * Copyright (c) 2002, Agere Systems
 * All rights reserved. 
 * 
 * Redistribution and use in source or binary forms, with or
 * without modifications, are permitted provided that the
 * following conditions are met: 
 * 
 * o Redistributions of source code must retain the above
 *   copyright notice, this list of conditions and the
 *   following Disclaimer in comments in the code as well as
 *   in the documentation and/or other materials provided with
 *   the distribution.
 * o Redistributions in binary form must reproduce the above
 *   copyright notice, this list of conditions and the following
 *   Disclaimer in the documentation and/or other materials
 *   provided with the distribution.
 * o Neither the name of Agere Systems nor the names of the
 *   contributors may be used to endorse or promote products
 *   derived from this software without specific prior written
 *   permission. 
 * 
 * Disclaimer 
 * 
 * THIS SOFTWARE IS PROVIDED "AS IS" AND ANY EXPRESS OR IMPLIED
 * WARRANTIES, INCLUDING, BUT NOT LIMITED TO, INFRINGEMENT AND THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR
 * PURPOSE ARE DISCLAIMED. ANY USE, MODIFICATION OR DISTRIBUTION OF
 * THIS SOFTWARE IS SOLELY AT THE USER'S OWN RISK. IN NO EVENT SHALL
 * AGERE SYSTEMS OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT,
 * INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING,
 * BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
 * LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER
 * CAUSED AND ON ANY THEORY OF LIABILITY, INCLUDING, BUT NOT LIMITED
 * TO, CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR
 * OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE,
 * EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE. 
 * 
 **********************************************************/

#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <strings.h>
#include <assert.h>

#include "fatal.h"
#include "list.h"
#include "file.h"
#include "registry.h"
#include "readGda.h"
#include "writeGda.h"

int main(int argc, char **argv) {
  char file[1000];
  int count = 0;
  FILE *fd;
  int i, k;
  char *output;
  int quiet = 0;

  if (argc < 2) goto usageProblem;
  
  output = "gdal.gda";

  initializeRegistry();

  for (i=1; i<argc; i++) {
    if (!strcmp(argv[i], "-f")) {
      if (++i >= argc) goto usageProblem;
      fd = fopen(argv[i], "r");
      if (!fd) {
	fprintf(stderr, "main: \"%s\" not found.\n", argv[i]);
      } else {
	while (fgets(file, 999, fd)) {
	  k = strlen(file) - 1;
	  if (file[k] == '\n') file[k] = 0;
	  count += readGda(file);
	  clearLocals();
	}
	fclose(fd);
      }
      continue;
    } 
    if (!strcmp(argv[i], "-o")) {
      if (++i < argc) output = argv[i]; else goto usageProblem;
      continue;
    } 
    if (!strcmp(argv[i], "-q")) {
      quiet = 1;
      continue;
    } 
    if (!strcmp(argv[i], "-d")) {
      debug = 1;
      continue;
    } 
    count += readGda(argv[i]);
    clearLocals();
  }
  
  if (!count) {
    fprintf(stderr, "main: no input. Aborted.\n");
    exit(1);
  }
  
  if (!quiet) {
    printf("Files linked:         %i\n", count);
    printf("Output file:          %s\n", output);
  }

  writeGda(output, !quiet);
  if(debug) stats();
  exit(0);
  
usageProblem:
  fprintf(stderr, "Usage: \"gdal [-d] [-q] [-o targetfile] [-f configfile] [sourcefile1] [sourcefile2] ...\"\n");
  fprintf(stderr, "Where: targetfile is the name of the output file (\"gdal.gda\" by default),\n");
  fprintf(stderr, "       sourcefile1 sourcefile2 ... are input files,\n");
  fprintf(stderr, "       configfile is a file containing the names of other input files.\n");
  exit(1);
  return 0;
}
