/**********************************************************
 * This software is provided subject to the following terms
 * and conditions, which you should read carefully before
 * using the software. Using this software indicates your
 * acceptance of these terms and conditions.  If you do not
 * agree with these terms and conditions, do not use the
 * software. 
 * 
 * Copyright (c) 2002, Agere Systems
 * All rights reserved. 
 * 
 * Redistribution and use in source or binary forms, with or
 * without modifications, are permitted provided that the
 * following conditions are met: 
 * 
 * o Redistributions of source code must retain the above
 *   copyright notice, this list of conditions and the
 *   following Disclaimer in comments in the code as well as
 *   in the documentation and/or other materials provided with
 *   the distribution.
 * o Redistributions in binary form must reproduce the above
 *   copyright notice, this list of conditions and the following
 *   Disclaimer in the documentation and/or other materials
 *   provided with the distribution.
 * o Neither the name of Agere Systems nor the names of the
 *   contributors may be used to endorse or promote products
 *   derived from this software without specific prior written
 *   permission. 
 * 
 * Disclaimer 
 * 
 * THIS SOFTWARE IS PROVIDED "AS IS" AND ANY EXPRESS OR IMPLIED
 * WARRANTIES, INCLUDING, BUT NOT LIMITED TO, INFRINGEMENT AND THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR
 * PURPOSE ARE DISCLAIMED. ANY USE, MODIFICATION OR DISTRIBUTION OF
 * THIS SOFTWARE IS SOLELY AT THE USER'S OWN RISK. IN NO EVENT SHALL
 * AGERE SYSTEMS OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT,
 * INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING,
 * BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
 * LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER
 * CAUSED AND ON ANY THEORY OF LIABILITY, INCLUDING, BUT NOT LIMITED
 * TO, CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR
 * OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE,
 * EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE. 
 * 
 **********************************************************/

#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <strings.h>
#include <assert.h>

#include "fatal.h"
#include "list.h"
#include "file.h"
#include "registry.h"
#include "writeGda.h"

#define VERSION 3
#define CALLSITE 0x82

#define programVar(var) (*(var)->name && *(var)->type)

static int phase;
static int pos;

static int writeGlobals(void) {
  struct list *ptr;
  struct variable *global;
  int count = 0;

  for(ptr=globalsList; ptr; ptr=ptr->next) {
    count++;
    global = ptr->elem;
    if(phase) {
      writeString(global->name);
      writeInt(global->newOffset);
    } else {
      pos += strlen(global->name) + 5;
    }
  }
  return count;
}

static int writeStaticDeps(void) {
  struct variable *var;
  struct dependency *dep;
  int stringOffset, i, count = 0;

  for(i=0; i<varCount(); i++) {
    var = varOfIndex(i);
    for(dep=var->staticDeps; dep; dep=dep->next) {
      count++;
      stringOffset = addString(dep->locString);
      if(phase) {
	writeChar(dep->dtype);
	writeInt(var->newOffset);
	writeInt(dep->variable->newOffset);
	writeInt(stringOffset);
	writeInt(dep->startLine);
	writeInt(dep->endLine);
      } else {
	pos += 21;
      }
    }
  }
  return count;
}

static int writeVariables(void) {
  struct variable *var;
  struct dependency *dep;
  int i, j, count = 0, depCount = 0, revCount = 0, programCount = 0;
  
  for(j=0; j<varCount(); j++) {
    var = varOfIndex(j);
    count++;
    if(programVar(var)) programCount++;
    if(!phase) {
      var->newOffset = pos;
      var->commonOffset = pos += 16; /* header */
      var->depOffset = pos += strlen(var->name) + strlen(var->type) + 18;
      addString(var->locString);

      if(var->prototype) {
	pos += 9 + 4 * var->prototype->nb;
      }

      for(dep=var->deps; dep; dep=dep->next) {
	pos += 17;
	if(dep->dtype != CALLSITE) pos += strlen(dep->stringL) + strlen(dep->stringR) + 2;
	addString(dep->locString);
      }

      var->revOffset = pos;

      for(dep=var->revDeps; dep; dep=dep->next) {
	pos += 17;
	if(dep->dtype != CALLSITE) pos += strlen(dep->stringL) + strlen(dep->stringR) + 2;
	addString(dep->locString);
      }
      
      var->endOffset = pos;
    } else {
      writeInt(var->commonOffset);
      writeInt(var->depOffset);
      writeInt(var->revOffset);
      writeInt(var->endOffset);
      writeString(var->name);
      writeInt(addString(var->locString));
      writeInt(var->startLine);
      writeInt(var->endLine);
      writeString(var->type);
      if(var->function) {
	writeInt(var->function->newOffset);
      } else {
	writeInt(0);
      }

      if(var->prototype) {
	writeChar(var->prototype->dtype);
	if(var->prototype->ret) {
	  writeInt(var->prototype->ret->newOffset);
	} else {
	  writeInt(0);
	}
	writeInt(var->prototype->nb);
	for(i=0; i<var->prototype->nb; i++) {
	  writeInt((*(((struct variable **)(var->prototype+1))+i))->newOffset);
	}
      }

      for(dep=var->deps; dep; dep=dep->next) {
	depCount++;

	writeChar(dep->dtype);
	writeInt(dep->variable->newOffset);
	writeInt(addString(dep->locString));
	writeInt(dep->startLine);
	writeInt(dep->endLine);
	if(dep->dtype != CALLSITE) {
	  writeString(dep->stringL);
	  writeString(dep->stringR);
	}
      }

      for(dep=var->revDeps; dep; dep=dep->next) {
	revCount++;
	writeChar(dep->dtype);
	writeInt(dep->variable->newOffset);
	writeInt(addString(dep->locString));
	writeInt(dep->startLine);
	writeInt(dep->endLine);
	if(dep->dtype != CALLSITE) {
	  writeString(dep->stringL);
	  writeString(dep->stringR);
	}
      }
    }
  }
  if(debug) fprintf(stderr, "depCount: %d\n", depCount);
  fprintf(stderr, "programCount: %d\n", programCount);
  return count;
}

static int writeHashTable(void) {
  struct variable *var;
  int hashSize, hash, i, j, count = 0;
  int *hashTable;
  char *name;

  for(hashSize=1; hashSize<2*varCount(); hashSize*=2);
  if(phase) {
    hashTable = (int *)CALLOC(hashSize, sizeof(int));

    for(i=0; i<varCount(); i++) {
      var = varOfIndex(i);
      name = strrchr(var->name, '.');
      if(name) name++; else name = var->name;
      hash = 0;
      for(j=0; name[j]; j++) {
	hash = ((hash * 67) + name[j]) % hashSize;
      }
      while(hashTable[hash]) {
	hash = (hash + 17) % hashSize;
      }
      hashTable[hash] = var->newOffset;
    }
    
    for(hash=0; hash<hashSize; hash++) {
      writeInt(hashTable[hash]);
      if(hashTable[hash]) count++;
    }
    free(hashTable);
  } else {
    pos += 4 * hashSize;
  }
  return hashSize;
}

static int writeStrings(void) {
  struct list *ptr;
  struct string *string;
  int count = 0;

  for(ptr=stringsList; ptr; ptr=ptr->next) {
    count++;
    string = (struct string *)ptr->elem;
    if(!phase) {
      string->offset = pos;
      pos += strlen(string->name) + 1;
    } else {
      writeString(string->name);
    }
  }
  return count;
}

static int writeFun(void) {
  struct list *ptr;
  int count = 0;

  for(ptr=funptr; ptr; ptr=ptr->next) {
    count++;
    if(!phase) {
      pos += 4;
    } else {
      writeInt(((struct variable *)ptr->elem)->newOffset);
    }
  }
  return count;
}

void writeGda(char *file, int verbose) {
  int version = VERSION;
  int globalOffset, globalLength;
  int staticOffset, staticLength;
  int stringsOffset, stringsLength;
  int dynamicOffset, dynamicLength;
  int hashOffset, hashLength;
  int funOffset, funLength;
  int count;

  pos = 0;
  phase = 0;

  pos += globalOffset = 60 + 8; /* header */
  if(debug) printf("globalOffset = %X\n", globalOffset);
  count = writeGlobals();
  if(verbose) printf("Global variables:     %i\n", count);
  globalLength = pos - globalOffset;
  if(debug) printf("globalLength = %X\n", globalLength);

  staticOffset = pos;
  if(debug) printf("staticOffset = %X\n", staticOffset);
  count = writeStaticDeps();
  if(verbose) printf("Static dependencies:  %i\n", count);
  staticLength = pos - staticOffset;
  if(debug) printf("staticLength = %X\n", staticLength);

  dynamicOffset = pos; 
  if(debug) printf("dynamicOffset = %X\n", dynamicOffset);
  count = writeVariables();
  if(verbose) printf("Variables:            %i\n", count);
  dynamicLength = pos - dynamicOffset;
  if(debug) printf("dynamicLength = %X\n", dynamicLength);

  hashOffset = pos;
  if(debug) printf("hashOffset = %X\n", hashOffset);
  count = writeHashTable();
  if(verbose) printf("Hashtable:            %i\n", count);
  hashLength = pos - hashOffset;
  if(debug) printf("hashLength = %X\n", hashLength);
  
  stringsOffset = pos;
  if(debug) printf("stringsOffset = %X\n", stringsOffset);
  count = writeStrings();
  if(verbose) printf("Strings:              %i\n", count);
  stringsLength = pos - stringsOffset;
  if(debug) printf("stringsLength = %X\n", stringsLength);

  funOffset = pos;
  if(debug) printf("funsOffset = %X\n", funOffset);
  count = writeFun();
  if(verbose) printf("Fun:                  %i\n", count);
  funLength = pos - funOffset;
  if(debug) printf("funLength = %X\n", funLength);

  if(debug) printf("fileLength = %X\n", pos);
  
  phase = 1;
  
  openFile(file);
  
  writeInt(version);
  writeInt(globalOffset);
  writeInt(globalLength);
  writeInt(staticOffset);
  writeInt(staticLength);
  writeInt(stringsOffset);
  writeInt(stringsLength);
  writeInt(hashOffset);
  writeInt(hashLength);
  writeInt(dynamicOffset);
  writeInt(dynamicLength);
  writeInt(0);
  writeInt(0);
  writeInt(0);
  writeInt(0);
  writeInt(funOffset);
  writeInt(funLength);

  writeGlobals();
  writeStaticDeps();
  writeVariables();
  writeHashTable();
  writeStrings();
  writeFun();

  closeFile();
}
