(**********************************************************
 * This software is provided subject to the following terms
 * and conditions, which you should read carefully before
 * using the software. Using this software indicates your
 * acceptance of these terms and conditions.  If you do not
 * agree with these terms and conditions, do not use the
 * software. 
 * 
 * Copyright (c) 2002, Agere Systems
 * All rights reserved. 
 * 
 * Redistribution and use in source or binary forms, with or
 * without modifications, are permitted provided that the
 * following conditions are met: 
 * 
 * o Redistributions of source code must retain the above
 *   copyright notice, this list of conditions and the
 *   following Disclaimer in comments in the code as well as
 *   in the documentation and/or other materials provided with
 *   the distribution.
 * o Redistributions in binary form must reproduce the above
 *   copyright notice, this list of conditions and the following
 *   Disclaimer in the documentation and/or other materials
 *   provided with the distribution.
 * o Neither the name of Agere Systems nor the names of the
 *   contributors may be used to endorse or promote products
 *   derived from this software without specific prior written
 *   permission. 
 * 
 * Disclaimer 
 * 
 * THIS SOFTWARE IS PROVIDED "AS IS" AND ANY EXPRESS OR IMPLIED
 * WARRANTIES, INCLUDING, BUT NOT LIMITED TO, INFRINGEMENT AND THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR
 * PURPOSE ARE DISCLAIMED. ANY USE, MODIFICATION OR DISTRIBUTION OF
 * THIS SOFTWARE IS SOLELY AT THE USER'S OWN RISK. IN NO EVENT SHALL
 * AGERE SYSTEMS OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT,
 * INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING,
 * BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
 * LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER
 * CAUSED AND ON ANY THEORY OF LIABILITY, INCLUDING, BUT NOT LIMITED
 * TO, CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR
 * OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE,
 * EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE. 
 * 
 **********************************************************)

structure readConfig =
  struct

    fun targetFormatMsg () =
      (print "Target Format Error: each target must be on a new line and have the form\n";
       print "                     x1.x2...xn<file:lineNumber>\n")

    type target = {name: string list, loc: (string * int option) option}

    exception FormatError

    fun cleanStr s =
      let val cl = String.explode s
	fun f (#"\n" :: cl) = f cl
	  | f(#"\t" :: cl) = f cl
	  | f(#" " :: cl) = f cl
	  | f(c :: cl) = c :: (f cl)
	  | f nil = nil
      in
	String.implode(f cl)
      end

    fun lastChar s c = (String.sub(s, (String.size s) - 1) = c)
    fun rmLast s = String.substring(s, 0, (String.size s)-1)

    fun readName nameStr = String.fields (fn x => (x = #".")) nameStr

    fun readLoc locStr =
      (case String.fields (fn x=> (x = #":")) locStr of
	 [file, lineNum] => (file, Int.fromString lineNum)
       | [file] => (file, NONE)
       | _ => raise FormatError)

    fun readLine str =
      (case String.fields (fn x => (x = #"<")) str of
	 [nameStr] => {name = readName nameStr, loc=NONE}
       | [nameStr, locStr] =>
	   let val locStr =
	     if lastChar locStr #">" then rmLast locStr
	     else
	       locStr
	   in
	     {name=readName nameStr, loc=SOME(readLoc locStr)}
	   end
       | _ => raise FormatError)

    fun readTargets file =
      let 
	val stream = TextIO.openIn file
	fun g() = if TextIO.endOfStream stream then nil
		  else ((readLine(cleanStr(TextIO.inputLine stream))) :: g())
      in
	g()
      end handle _ => (print ("Could not read file " ^ file ^ "\n");
		       nil)

    fun getFileNames stream =
      let 
	fun g (cur, sofar, c :: cl) =
	  let fun newF() = if cur = nil then g(nil, sofar, cl)
			   else g(nil, (List.rev cur) :: sofar, cl)
	  in
	    case c of
	      #"\n" => newF()
	    | #"\t" => newF()
	    | #" " => newF()
	    | _ => g(c :: cur, sofar, cl)
	  end
	  | g (cur, sofar, nil) = 
	  if TextIO.endOfStream stream then 
	    if cur = nil then List.rev sofar
	    else List.rev((List.rev cur) :: sofar)
	  else
	    g(cur, sofar, String.explode (TextIO.inputLine stream))
      in
	List.map String.implode (g(nil, nil, nil))
      end

    fun readConfig file =
      let val stream = TextIO.openIn file
      in
	getFileNames stream
      end handle _ => (print ("Could not read file " ^ file ^ "\n");
		       nil)
  end
