(**********************************************************
 * This software is provided subject to the following terms
 * and conditions, which you should read carefully before
 * using the software. Using this software indicates your
 * acceptance of these terms and conditions.  If you do not
 * agree with these terms and conditions, do not use the
 * software. 
 * 
 * Copyright (c) 2002, Agere Systems
 * All rights reserved. 
 * 
 * Redistribution and use in source or binary forms, with or
 * without modifications, are permitted provided that the
 * following conditions are met: 
 * 
 * o Redistributions of source code must retain the above
 *   copyright notice, this list of conditions and the
 *   following Disclaimer in comments in the code as well as
 *   in the documentation and/or other materials provided with
 *   the distribution.
 * o Redistributions in binary form must reproduce the above
 *   copyright notice, this list of conditions and the following
 *   Disclaimer in the documentation and/or other materials
 *   provided with the distribution.
 * o Neither the name of Agere Systems nor the names of the
 *   contributors may be used to endorse or promote products
 *   derived from this software without specific prior written
 *   permission. 
 * 
 * Disclaimer 
 * 
 * THIS SOFTWARE IS PROVIDED "AS IS" AND ANY EXPRESS OR IMPLIED
 * WARRANTIES, INCLUDING, BUT NOT LIMITED TO, INFRINGEMENT AND THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR
 * PURPOSE ARE DISCLAIMED. ANY USE, MODIFICATION OR DISTRIBUTION OF
 * THIS SOFTWARE IS SOLELY AT THE USER'S OWN RISK. IN NO EVENT SHALL
 * AGERE SYSTEMS OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT,
 * INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING,
 * BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
 * LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER
 * CAUSED AND ON ANY THEORY OF LIABILITY, INCLUDING, BUT NOT LIMITED
 * TO, CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR
 * OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE,
 * EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE. 
 * 
 **********************************************************)

structure Types =
  struct
    type loc = SourceMap.location
    type ctype = Ast.ctype
    type uid = int
    type vid = {pid: uid, loc: loc, name: string, tp: ctype, global: bool}
      
    datatype var =
      A of {pid: uid, loc: loc}
    | F of {pid: uid, var: var, arg: int}
    | R of {pid: uid, var: var}
    | V of vid
      
    datatype depCategory = NOTHING | STRONG | STRONG_LVAL | WEAK
      
    datatype dynDep =
      SimpleDep of {dcat: depCategory, loc: loc,
		    lstr: string, read: var, readDirect: bool,
		    rstr: string, write: var, writeDirect: bool}
    | CallSite of {caller: vid, calle: var, loc:loc}
       
    datatype staticDep =
      Addrof of {loc: loc, lval: vid, write: var}
      
    type funInfos = {ptr: bool, ret: var option, args: var list}
      
    type varInfos = {var: var, used: bool,
		     dep: dynDep list, rev: dynDep list,
		     context: vid option, proto: funInfos option}
      
    structure UidMap = HashTableFn (struct
				      type hash_key = uid
				      fun hashVal uid = Word.fromInt uid
				      fun sameKey (uid1, uid2) = (uid1 = uid2)
				    end)
      
    fun ctypeToString tidtab ctype =
      (PPLib.ppToString (PPAst.ppCtype () tidtab) ctype)

    fun varToPid (A {pid, ...}) = pid
      | varToPid (F {pid, ...}) = pid
      | varToPid (R {pid, ...}) = pid
      | varToPid (V {pid, ...}) = pid
  end
