/**********************************************************
 * This software is provided subject to the following terms
 * and conditions, which you should read carefully before
 * using the software. Using this software indicates your
 * acceptance of these terms and conditions.  If you do not
 * agree with these terms and conditions, do not use the
 * software. 
 * 
 * Copyright (c) 2002, Agere Systems
 * All rights reserved. 
 * 
 * Redistribution and use in source or binary forms, with or
 * without modifications, are permitted provided that the
 * following conditions are met: 
 * 
 * o Redistributions of source code must retain the above
 *   copyright notice, this list of conditions and the
 *   following Disclaimer in comments in the code as well as
 *   in the documentation and/or other materials provided with
 *   the distribution.
 * o Redistributions in binary form must reproduce the above
 *   copyright notice, this list of conditions and the following
 *   Disclaimer in the documentation and/or other materials
 *   provided with the distribution.
 * o Neither the name of Agere Systems nor the names of the
 *   contributors may be used to endorse or promote products
 *   derived from this software without specific prior written
 *   permission. 
 * 
 * Disclaimer 
 * 
 * THIS SOFTWARE IS PROVIDED "AS IS" AND ANY EXPRESS OR IMPLIED
 * WARRANTIES, INCLUDING, BUT NOT LIMITED TO, INFRINGEMENT AND THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR
 * PURPOSE ARE DISCLAIMED. ANY USE, MODIFICATION OR DISTRIBUTION OF
 * THIS SOFTWARE IS SOLELY AT THE USER'S OWN RISK. IN NO EVENT SHALL
 * AGERE SYSTEMS OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT,
 * INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING,
 * BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
 * LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER
 * CAUSED AND ON ANY THEORY OF LIABILITY, INCLUDING, BUT NOT LIMITED
 * TO, CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR
 * OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE,
 * EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE. 
 * 
 **********************************************************/

/* dtypes */
#define DEP 0x1
#define STRONG 0x2
#define DIRECT 0x4
#define READDIRECT 0x8
#define WRITEDIRECT 0x10
#define TRANSFERLVALS 0x20
#define PROTOTYPE 0x80
#define FUNPTR 0x81
#define CALLSITE 0x82

#define CMPBITS (DEP | STRONG | DIRECT)


/* about dtypes */
#define isDep(x) ((x) & DEP)
#define isStrong(x) ((x) & STRONG)
#define isWeak(x) (!(isStrong(x)))
#define isDirect(x) ((x) & DIRECT)
#define isViaPointer(x) (!(isDirect(x)))
#define isReadPtr(x) (!((x) & READDIRECT))
#define isWritePtr(x) (!((x) & WRITEDIRECT))
#define transferLvals(x) ((x) & TRANSFERLVALS)
#define isPrototype(x) ((x) & 0x80)
#define isFunPtr(x) ((x) == FUNPTR)


struct location {
  char *file;
  int startLine;
  int endLine;
};

struct varInfos {
  char *name;
  char *ctype;
  struct location loc;
  int function;
};

struct varOffsets {
  int prototype;
  int forward;
  int backward;
  int end;
};

struct depInfos {
  unsigned char dtype;
  int var;
  struct location loc;
  char *lstr;
  char *rstr;
};

struct staticInfos {
  unsigned char dtype;
  int var;
  int lval;
  struct location loc;
};

void map(char *file);

void getVarInfos(int offset, struct varInfos *infos);
void getVarOffsets(int offset, struct varOffsets *offsets);

unsigned char getProtoInfos(int offset, struct list *offsets);

int getDepInfos(int offset, struct depInfos *infos);
int getStaticInfos(int offset, struct staticInfos *infos);

int findTarget(char *name, struct varInfos *infos);

extern int counter; /* nb of calls to getDepInfos */


int funPtrCount(void);
int funPtrOfIndex(int n);
