/**********************************************************
 * This software is provided subject to the following terms
 * and conditions, which you should read carefully before
 * using the software. Using this software indicates your
 * acceptance of these terms and conditions.  If you do not
 * agree with these terms and conditions, do not use the
 * software. 
 * 
 * Copyright (c) 2002, Agere Systems
 * All rights reserved. 
 * 
 * Redistribution and use in source or binary forms, with or
 * without modifications, are permitted provided that the
 * following conditions are met: 
 * 
 * o Redistributions of source code must retain the above
 *   copyright notice, this list of conditions and the
 *   following Disclaimer in comments in the code as well as
 *   in the documentation and/or other materials provided with
 *   the distribution.
 * o Redistributions in binary form must reproduce the above
 *   copyright notice, this list of conditions and the following
 *   Disclaimer in the documentation and/or other materials
 *   provided with the distribution.
 * o Neither the name of Agere Systems nor the names of the
 *   contributors may be used to endorse or promote products
 *   derived from this software without specific prior written
 *   permission. 
 * 
 * Disclaimer 
 * 
 * THIS SOFTWARE IS PROVIDED "AS IS" AND ANY EXPRESS OR IMPLIED
 * WARRANTIES, INCLUDING, BUT NOT LIMITED TO, INFRINGEMENT AND THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR
 * PURPOSE ARE DISCLAIMED. ANY USE, MODIFICATION OR DISTRIBUTION OF
 * THIS SOFTWARE IS SOLELY AT THE USER'S OWN RISK. IN NO EVENT SHALL
 * AGERE SYSTEMS OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT,
 * INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING,
 * BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
 * LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER
 * CAUSED AND ON ANY THEORY OF LIABILITY, INCLUDING, BUT NOT LIMITED
 * TO, CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR
 * OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE,
 * EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE. 
 * 
 **********************************************************/

#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <strings.h>
#include <assert.h>

#include "fatal.h"
#include "hashtable.h"

#define STRIDE 17

struct hashtable {
  int size;     /* current size of the hashtable */
  int entries;  /* nb of objects registered */
  void **table;
};

struct hashtable *createTable(int power) {
  struct hashtable *table;
  
  table = (struct hashtable *)MALLOC(sizeof(struct hashtable));
  table->size = 1 << power;
  table->entries = 0;
  table->table = (void **)CALLOC(table->size, sizeof(void *));
  return table;
}

void clearTable(struct hashtable *table) {
  assert(table);

  bzero(table->table, table->size * sizeof(void *));
  table->entries = 0;
}

void freeTable(struct hashtable *table) {
  assert(table);

  free(table->table);
  free(table);
}

int hashSize(struct hashtable *table) {
  assert(table);

  return table->size;
}

#define EXPAND(num) \
static void expandTable##num(struct hashtable *table) { \
  void **newTable; \
  int newSize, h, i; \
\
  newSize = 2 * table->size; \
  newTable = (void **)CALLOC(newSize, sizeof(void *)); \
\
  for(i=0; i<table->size; i++) { \
    if(table->table[i]) { \
      HASH##num(&h, CAST##num(table->table[i]), newSize-1); \
      while(newTable[h]) h = (h+STRIDE) & (newSize-1); \
      newTable[h] = table->table[i]; \
    } \
  } \
\
  free(table->table); \
  table->table = newTable; \
  table->size = newSize; \
} \
\
void *findEntry##num(struct hashtable *table, TYPE##num key, void *(*buildEntry) (TYPE##num)) { \
  void **t; \
  int h, m; \
\
  assert(table); \
\
  if(buildEntry && 2*table->entries >= table->size) expandTable##num(table); \
\
  t = table->table; \
  m = table->size - 1; \
\
  HASH##num(&h, key, m); \
  while(t[h] && COMP##num(CAST##num(t[h]), key)) { \
    h = (h+STRIDE) & m; \
  } \
\
  if(!t[h] && buildEntry) { \
    t[h] = buildEntry(key); \
    table->entries++; \
  } \
\
  return t[h]; \
}

struct string {
  char *name;
  int offset;
};

#define TYPE1 int
#define TYPE2 char*
#define TYPE3 char*
#define TYPE4 int

#define HASH1(key, entry, mask) (*key = (entry) & (mask))
#define HASH2(key, entry, mask) (*key = (int)(entry) & (mask))
#define HASH3(key, entry, mask) { char *s; char c; *key = 0; for(s=entry; c=*s; s++) *key = (*key*67 + c) & mask; }
#define HASH4(key, entry, mask) (*key = (entry) & (mask))

#define CAST1(x) (*((int *)(x)))
#define CAST2(x) (*((char **)(x)))
#define CAST3(x) ((char *)(x))
#define CAST4(x) (((struct string *)(x))->offset)

#define COMP1(x, y) ((x) != (y))
#define COMP2(x, y) ((x) != (y))
#define COMP3(x, y) (strcmp((x), (y)))
#define COMP4(x, y) ((x) != (y))

EXPAND(1)
EXPAND(2)
EXPAND(3)
EXPAND(4)
