/**********************************************************
 * This software is provided subject to the following terms
 * and conditions, which you should read carefully before
 * using the software. Using this software indicates your
 * acceptance of these terms and conditions.  If you do not
 * agree with these terms and conditions, do not use the
 * software. 
 * 
 * Copyright (c) 2002, Agere Systems
 * All rights reserved. 
 * 
 * Redistribution and use in source or binary forms, with or
 * without modifications, are permitted provided that the
 * following conditions are met: 
 * 
 * o Redistributions of source code must retain the above
 *   copyright notice, this list of conditions and the
 *   following Disclaimer in comments in the code as well as
 *   in the documentation and/or other materials provided with
 *   the distribution.
 * o Redistributions in binary form must reproduce the above
 *   copyright notice, this list of conditions and the following
 *   Disclaimer in the documentation and/or other materials
 *   provided with the distribution.
 * o Neither the name of Agere Systems nor the names of the
 *   contributors may be used to endorse or promote products
 *   derived from this software without specific prior written
 *   permission. 
 * 
 * Disclaimer 
 * 
 * THIS SOFTWARE IS PROVIDED "AS IS" AND ANY EXPRESS OR IMPLIED
 * WARRANTIES, INCLUDING, BUT NOT LIMITED TO, INFRINGEMENT AND THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR
 * PURPOSE ARE DISCLAIMED. ANY USE, MODIFICATION OR DISTRIBUTION OF
 * THIS SOFTWARE IS SOLELY AT THE USER'S OWN RISK. IN NO EVENT SHALL
 * AGERE SYSTEMS OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT,
 * INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING,
 * BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
 * LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER
 * CAUSED AND ON ANY THEORY OF LIABILITY, INCLUDING, BUT NOT LIMITED
 * TO, CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR
 * OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE,
 * EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE. 
 * 
 **********************************************************/

struct dependency {
  int offset;                    /* offset */
  struct variable *writeVar;     /* variable being written */
  struct variable *readVar;      /* variable being read */
  unsigned char dtype;           /* type */
};

struct variable {
  int offset;                    /* offset */
  struct chain *chain;           /* current dependency chain info */
  struct set *lvals;             /* set of lvals this variable may take */
  struct set *tracks;            /* set of lvals this variable may take */
  struct chain *lvalsChain;      /* current dependency chain info for lvals */

  struct set *initialLvals;
  struct list *initialPtrs;

  unsigned int demand : 1;
  unsigned int track : 1;

  unsigned int initialized : 1;  /* already initialized ? */
  unsigned int prototype : 1;    /* prototype already loaded ? */
  unsigned int nonTarget : 1;    /* non target var ? */
};

struct chain {
  int offset;                    /* offset of the first dep of the chain */
  struct variable *writeVar;     /* variable the chain is about */
  struct variable *readVar;      /* next variable in chain */
  unsigned char dtype;           /* dtype of chain */
  unsigned int changed : 1;      /* delayed propagatation ? */
  int length : 23;               /* length of the full chain */
};

void initialize(void);
void saveState(void);
void restoreState(void);

struct variable *varOfIndex(int index);
struct variable *varAtOffset(int offset);
int varIndex(struct variable *var);
int varCount(void);

struct dependency *depOfIndex(int index);
struct dependency *newDep(int offset, struct variable *writeVar, struct variable *readVar, unsigned char dtype);
int depIndex(struct dependency *dep);
int depCount(void);

struct chain *chainOfIndex(int index);
struct chain *newChain(void);
int chainIndex(struct chain *chain);
int chainCount(void);

struct list *protoOfVar(struct variable *var, struct list *proto);

#define isInitialized(v) ((v)->initialized)
#define isTargetVar(v) ((v)->chain && !(v)->chain->length)
#define isNonTargetVar(v) ((v)->nonTarget)
#define hasDep(v) ((v)->chain != 0)
#define hasNoLvals(v) (isEmptySet(v->lvals))
#define hasLvalsDep(v) ((v)->lvalsChain != 0)

