(**********************************************************
 * This software is provided subject to the following terms
 * and conditions, which you should read carefully before
 * using the software. Using this software indicates your
 * acceptance of these terms and conditions.  If you do not
 * agree with these terms and conditions, do not use the
 * software. 
 * 
 * Copyright (c) 2002, Agere Systems
 * All rights reserved. 
 * 
 * Redistribution and use in source or binary forms, with or
 * without modifications, are permitted provided that the
 * following conditions are met: 
 * 
 * o Redistributions of source code must retain the above
 *   copyright notice, this list of conditions and the
 *   following Disclaimer in comments in the code as well as
 *   in the documentation and/or other materials provided with
 *   the distribution.
 * o Redistributions in binary form must reproduce the above
 *   copyright notice, this list of conditions and the following
 *   Disclaimer in the documentation and/or other materials
 *   provided with the distribution.
 * o Neither the name of Agere Systems nor the names of the
 *   contributors may be used to endorse or promote products
 *   derived from this software without specific prior written
 *   permission. 
 * 
 * Disclaimer 
 * 
 * THIS SOFTWARE IS PROVIDED "AS IS" AND ANY EXPRESS OR IMPLIED
 * WARRANTIES, INCLUDING, BUT NOT LIMITED TO, INFRINGEMENT AND THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR
 * PURPOSE ARE DISCLAIMED. ANY USE, MODIFICATION OR DISTRIBUTION OF
 * THIS SOFTWARE IS SOLELY AT THE USER'S OWN RISK. IN NO EVENT SHALL
 * AGERE SYSTEMS OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT,
 * INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING,
 * BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
 * LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER
 * CAUSED AND ON ANY THEORY OF LIABILITY, INCLUDING, BUT NOT LIMITED
 * TO, CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR
 * OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE,
 * EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE. 
 * 
 **********************************************************)

structure Output =
  struct
    val pos = ref 0 (* current position in the stream *)
    val outStream = (ref NONE): TextIO.outstream option ref (* ouput stream *)
      
    fun writeChar c =
      (pos := (!pos) + 1;
       case !outStream of
	 SOME stream => TextIO.output1 (stream, c)
       | NONE => ())
      
    fun writeString s =
      (pos := (!pos) + (String.size s) + 1;
       case !outStream of
	 SOME stream => (TextIO.output (stream, s);
			 TextIO.output1 (stream, chr 0))
       | NONE => ())
      
    fun writeInt i =
      let
	val w = Word32.fromInt i
	val w8 = Word.fromInt 8
	val w16 = Word.fromInt 16
	val w24 = Word.fromInt 24
	val b0 = Word32.>> (Word32.<< (w, w24), w24)
	val b1 = Word32.>> (Word32.<< (w, w16), w24)
	val b2 = Word32.>> (Word32.<< (w, w8), w24)
	val b3 = Word32.>> (w, w24)
      in
	writeChar (Char.chr (Word32.toInt b0));
	writeChar (Char.chr (Word32.toInt b1));
	writeChar (Char.chr (Word32.toInt b2));
	writeChar (Char.chr (Word32.toInt b3))
      end
  end
